@@ -1,5 +1,7 @@
 import copy
 import math
+import os
+import sys
 import random
 import string
 import warnings
@@ -15,6 +17,18 @@ from sklearn.ensemble import RandomForestClassifier
 from pyexplainer.rulefit import RuleFit
 
 
+def get_base_prefix_compat():
+    """Get base/real prefix, or sys.prefix if there is none."""
+    return getattr(sys, "base_prefix", None) or getattr(sys, "real_prefix", None) or sys.prefix
+
+
+def in_virtualenv():
+    return get_base_prefix_compat() != sys.prefix
+
+
+INSIDE_VIRTUAL_ENV = in_virtualenv()
+
+
 def data_validation(data):
     """Validate that the given data format is a list of dictionary.
 
@@ -41,6 +55,50 @@ def data_validation(data):
     return valid
 
 
+def get_default_data_and_model():
+    this_dir, _ = os.path.split(__file__)
+    path_train = this_dir + "/default_data/activemq-5.0.0.zip"
+    path_test = this_dir + "/default_data/activemq-5.1.0.zip"
+    if INSIDE_VIRTUAL_ENV:
+        cwd = os.getcwd()
+        path_train = cwd + "/tests/pyexplainer_test_data/activemq-5.0.0.zip"
+        path_test = cwd + "/tests/pyexplainer_test_data/activemq-5.1.0.zip"
+    training_data = pd.read_csv(path_train, index_col='File')
+    dependent_vars = training_data.columns[-4]
+    selected_features = ["ADEV", "AvgCyclomaticModified", "AvgEssential", "AvgLineBlank", "AvgLineComment",
+                         "CountClassBase", "CountClassCoupled", "CountClassDerived", "CountDeclClass",
+                         "CountDeclClassMethod", "CountDeclClassVariable", "CountDeclInstanceVariable",
+                         "CountDeclMethodDefault", "CountDeclMethodPrivate", "CountDeclMethodProtected",
+                         "CountDeclMethodPublic", "CountInput_Mean", "CountInput_Min", "CountOutput_Min", "MAJOR_LINE",
+                         "MaxInheritanceTree", "MaxNesting_Min", "MINOR_COMMIT", "OWN_COMMIT", "OWN_LINE",
+                         "PercentLackOfCohesion", "RatioCommentToCode"]
+    all_cols = training_data.columns
+    for col in all_cols:
+        if col not in selected_features:
+            all_cols = all_cols.drop(col)
+    independent_vars = all_cols
+    X_train = training_data.loc[:, independent_vars]
+    y_train = training_data.loc[:, dependent_vars]
+
+    blackbox_model = RandomForestClassifier(max_depth=3, random_state=0)
+    blackbox_model.fit(X_train, y_train)
+
+    testing_data = pd.read_csv(path_test, index_col='File')
+    X_test = testing_data.loc[:, independent_vars]
+    y_test = testing_data.loc[:, dependent_vars]
+
+    sample_explain_index = 0
+    X_explain = X_test.iloc[[sample_explain_index]]
+    y_explain = y_test.iloc[[sample_explain_index]]
+    return {'X_train': X_train,
+            'y_train': y_train,
+            'indep': independent_vars,
+            'dep': dependent_vars,
+            'blackbox_model': blackbox_model,
+            'X_explain': X_explain,
+            'y_explain': y_explain}
+
+
 def id_generator(size=15, random_state=check_random_state(None)):
     """Generate unique ids for div tag which will contain the visualisation stuff from d3.
 
@@ -136,7 +194,6 @@ class PyExplainer:
         else:
             print("dep (label column name) should be type 'str'")
             raise TypeError
-        # todo- validate blackbox model
         if isinstance(blackbox_model, sklearn.ensemble.RandomForestClassifier):
             self.blackbox_model = blackbox_model
         else:
@@ -173,8 +230,8 @@ class PyExplainer:
                 X_explain,
                 y_explain,
                 top_k=3,
-                max_rules=10,
-                max_iter=10,
+                max_rules=2000,
+                max_iter=10000,
                 cv=5,
                 search_function='CrossoverInterpolation',
                 debug=False):
@@ -217,14 +274,14 @@ class PyExplainer:
         >>> blackbox_model = RandomForestClassifier(max_depth=3, random_state=0)
         >>> blackbox_model.fit(X_train, y_train)
         >>> class_label = ['Clean', 'Defect']
-        >>> pyExp = PyExplainer(X_train, y_train, indep, dep, class_label, blackbox_model)
+        >>> py_explainer = PyExplainer(X_train, y_train, indep, dep, class_label, blackbox_model)
         >>> sample_test_data = pd.read_csv('../tests/pyexplainer_test_data/activemq-5.0.0.csv', index_col = 'File')
         >>> X_test = sample_test_data.loc[:, indep]
         >>> y_test = sample_test_data.loc[:, dep]
         >>> sample_explain_index = 0
         >>> X_explain = X_test.iloc[[sample_explain_index]]
         >>> y_explain = y_test.iloc[[sample_explain_index]]
-        >>> pyExp.explain(X_explain, y_explain, search_function = 'crossoverinterpolation', top_k = 3, max_rules=30, max_iter =5, cv=5, debug = False)
+        >>> py_explainer.explain(X_explain, y_explain, search_function = 'crossoverinterpolation', top_k = 3, max_rules=30, max_iter =5, cv=5, debug = False)
         """
         # check if X_explain is a DF
         if not isinstance(X_explain, pd.core.frame.DataFrame):
@@ -244,7 +301,7 @@ class PyExplainer:
             synthetic_object = self.generate_instance_crossover_interpolation(X_explain, y_explain, debug=debug)
         elif search_function.lower() == 'randomperturbation':
             # This random perturbation approach to generate instances is used by LIME to gerate synthetic instances
-            synthetic_object = self.generate_instance_random_perturbation(X_explain=X_explain,debug=debug)
+            synthetic_object = self.generate_instance_random_perturbation(X_explain=X_explain, debug=debug)
 
         # Step 2 - Generate predictions of synthetic instances using the global model
         synthetic_instances = synthetic_object['synthetic_data'].loc[:, self.indep]
@@ -298,7 +355,8 @@ class PyExplainer:
                     'indep': self.indep,
                     'dep': self.dep,
                     'top_k_positive_rules': top_k_positive_rules,
-                    'top_k_negative_rules': top_k_negative_rules}
+                    'top_k_negative_rules': top_k_negative_rules,
+                    'local_rulefit_model': local_rulefit_model}
         return rule_obj
 
     def generate_bullet_data(self, parsed_rule_object):
@@ -308,8 +366,6 @@ class PyExplainer:
         ----------
         parsed_rule_object : :obj:`dict`
             Top rules parsed from Rule object.
-        X_explain : :obj:`pandas.core.frame.DataFrame`
-            Explained Dataframe generated from RuleFit model.
 
         Returns
         -------
@@ -330,8 +386,8 @@ class PyExplainer:
             tmp_threshold_value = round(float(tmp_rule['value']), 2)
             tmp_actual_value = round(X_explain[tmp_rule['variable']][0], 2)
             tmp_markers = [tmp_actual_value]
-            plot_min = int(round(max(tmp_min, tmp_threshold_value - tmp_interval), 0)) * 1.0
-            plot_max = int(round(min(tmp_max, tmp_threshold_value + tmp_interval), 0)) * 1.0
+            plot_min = int(round(max(tmp_min, tmp_threshold_value - tmp_interval), 0))  # * 1.0
+            plot_max = int(round(min(tmp_max, tmp_threshold_value + tmp_interval), 0))  # * 1.0
 
             # keep marker in the range
             if tmp_markers[0] > plot_max:
@@ -349,9 +405,14 @@ class PyExplainer:
                 tmp_step = [1]
 
             bullet_total_width = 450
-            tmp_startpoints = [0, round((tmp_threshold_value - plot_min) / diff_plot_max_min * bullet_total_width, 0)]
-            tmp_widths = [round((tmp_threshold_value - plot_min) / diff_plot_max_min * bullet_total_width, 0),
-                          round((plot_max - tmp_threshold_value) / diff_plot_max_min * bullet_total_width, 0)]
+
+            tmp_start_points = [0, round((tmp_threshold_value - plot_min) / diff_plot_max_min * bullet_total_width
+                                         if diff_plot_max_min * bullet_total_width else 0, 0)]
+
+            tmp_widths = [round((tmp_threshold_value - plot_min) / diff_plot_max_min * bullet_total_width
+                                if diff_plot_max_min * bullet_total_width else 0, 0),
+                          round((plot_max - tmp_threshold_value) / diff_plot_max_min * bullet_total_width
+                                if diff_plot_max_min * bullet_total_width else 0, 0)]
 
             id = '#' + str(i + 1)
             var_name = str(tmp_rule['variable'])
@@ -375,7 +436,7 @@ class PyExplainer:
                 "subtitle": tmp_subtitle_text,
                 "ticks": tmp_ticks,
                 "step": tmp_step,
-                "startPoints": tmp_startpoints,
+                "startPoints": tmp_start_points,
                 "widths": tmp_widths,
                 "colors": tmp_colors,
                 "markers": tmp_markers,
@@ -391,22 +452,26 @@ class PyExplainer:
         :obj:`str`
             html String
         """
+        this_dir, _ = os.path.split(__file__)
+        with open(os.path.join(this_dir, 'css/styles.css'), encoding="utf8") as f:
+            style_css = f.read()
+        with open(os.path.join(this_dir, 'js/d3.min.js'), encoding="utf8") as f:
+            d3_js = f.read()
+        with open(os.path.join(this_dir, 'js/bullet.js'), encoding="utf8") as f:
+            bullet_js = f.read()
 
-        css_filepath = "css/styles.css"
         css_stylesheet = """
-            <link rel="stylesheet" href="%s" />
-        """ % css_filepath
+        <style>%s</style>
+        """ % style_css
 
-        d3_filepath = "js/d3.min.js"
-        bulletjs_filepath = "js/bullet.js"
         d3_script = """
-        <script src="%s"></script>
-        <script src="%s"></script>
-        """ % (d3_filepath, bulletjs_filepath)
+        <script>%s</script>
+        <script>%s</script>
+        """ % (d3_js, bullet_js)
 
         main_title = "What to do to decrease the risk of having defects?"
         title = """
-        <div style="position: relative; top: 0; width: 100vw; left: 27vw;">
+        <div style="position: relative; top: 0; width: 100vw; left: 20vw;">
             <b>%s</b>
         </div>
         """ % main_title
@@ -520,8 +585,7 @@ class PyExplainer:
         Returns
         -------
         :obj:`dict`
-            A dict with two keys 'synthetic_data' and 'sampled_class_frequency' generated via
-            Crossover and Interpolation.
+            A dict with two keys 'synthetic_data' and 'sampled_class_frequency' generated via Crossover and Interpolation.
         """
         categorical_vars = []
 
@@ -695,9 +759,22 @@ class PyExplainer:
             return {'synthetic_data': new_df_case,
                     'sampled_class_frequency': sampled_class_frequency}
 
-    # todo - documentation
     def generate_instance_random_perturbation(self, X_explain, debug=False):
-        """The random perturbation approach to generate synthetic instances which is also used by LIME."""
+        """The random perturbation approach to generate synthetic instances which is also used by LIME.
+
+        Parameters
+        ----------
+        X_explain : :obj:`pandas.core.frame.DataFrame`
+            X_explain (Testing Features)
+        debug : :obj:`bool`
+            True for debugging mode, False otherwise.
+
+        Returns
+        -------
+        :obj:`dict`
+            A dict with two keys 'synthetic_data' and 'sampled_class_frequency' generated via Random Perturbation.
+
+        """
         random_seed = 0
         data_row = X_explain.loc[:, self.indep].values
         num_samples = 1000
@@ -877,7 +954,7 @@ class PyExplainer:
         """
         slider_widgets = []
         data = self.__get_bullet_data()
-        style = {'description_width': '40%', 'widget_width': '60%'}
+        style = {'description_width': '40%'}
         layout = widgets.Layout(width='99%', height='20px')
 
         for d in data:
@@ -980,10 +1057,7 @@ class PyExplainer:
         :obj:`dict`
             A dict containing two keys, 'top_tofollow_rules' and 'top_toavoid_rules'
         """
-        if len(top_k_positive_rules) < len(top_k_negative_rules):
-            smaller_top_rule = len(top_k_positive_rules)
-        else:
-            smaller_top_rule = len(top_k_negative_rules)
+        smaller_top_rule = min([len(top_k_positive_rules), len(top_k_negative_rules)])
         if self.get_top_k_rules() > smaller_top_rule:
             self.set_top_k_rules(smaller_top_rule)
 
